"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lowdb_1 = tslib_1.__importDefault(require("lowdb"));
var plasma_cash_tx_1 = require("./plasma-cash-tx");
var bn_js_1 = tslib_1.__importDefault(require("bn.js"));
var PlasmaDB = /** @class */ (function () {
    function PlasmaDB(ethereum, dappchain, plasmaAddress, privateKey) {
        // If we're on node.js
        var adapter;
        if (typeof localStorage === 'undefined' || localStorage === null) {
            var FileSync = require('lowdb/adapters/FileSync');
            adapter = new FileSync("db/db_" + privateKey + ".json");
        }
        else {
            var LocalStorage = require('lowdb/adapters/LocalStorage');
            adapter = new LocalStorage('db');
        }
        this.db = lowdb_1.default(adapter);
        // Initialize the database
        this.db
            .defaults({
            ethereum: ethereum,
            dappchain: dappchain,
            plasma: plasmaAddress,
            privatekey: privateKey,
            coins: []
        })
            .write();
        console.log('Initialized database', this.db.value());
    }
    PlasmaDB.prototype.receiveCoin = function (coinId, block, tx) {
        // Find the coin in the database and add the block/proof.
        // Throw for duplicate block
        if (this.exists(coinId, block)) {
            return;
        }
        // @ts-ignore
        tx.proofBytes = Array.from(tx.proofBytes);
        // @ts-ignore
        tx.sigBytes = Array.from(tx.sigBytes);
        // Append for new coinId
        var result = this.db
            .get('coins')
            .push({
            slot: coinId,
            block: block,
            tx: tx
        })
            .write();
        // console.log('State updated', result)
    };
    PlasmaDB.prototype.getTx = function (coinId, block) {
        var result = this.db
            .get('coins')
            .filter({ slot: coinId, block: block })
            .value();
        var tx = result[0].tx;
        return new plasma_cash_tx_1.PlasmaCashTx({
            slot: new bn_js_1.default(tx.slot, 16),
            prevBlockNum: new bn_js_1.default(tx.prevBlockNum, 16),
            denomination: new bn_js_1.default(1),
            newOwner: tx.newOwner,
            prevOwner: tx.prevOwner,
            sig: Uint8Array.from(tx.sigBytes),
            proof: Uint8Array.from(tx.proofBytes)
        });
    };
    PlasmaDB.prototype.exists = function (coinId, block) {
        var result = this.db
            .get('coins')
            .filter({ slot: coinId, block: block })
            .value();
        if (result.length > 0) {
            console.log("Transaction: " + coinId + " at Block: " + block + " already exists in local state");
            return true;
        }
        else {
            return false;
        }
    };
    PlasmaDB.prototype.removeCoin = function (coinId) {
        this.db
            .get('coins')
            .remove({ slot: coinId })
            .write();
        console.log("Coin " + coinId + " removed");
    };
    PlasmaDB.prototype.getCoin = function (coinId) {
        return this.db
            .get('coins')
            .filter({ slot: coinId })
            .value();
    };
    PlasmaDB.prototype.getAllCoins = function () {
        return this.db.get('coins').value();
    };
    return PlasmaDB;
}());
exports.default = PlasmaDB;
// Example -> transform in a test
// const db = new PlasmaDB('localhost:8545', 'localhost:46658', '0x1234', '0x6666')
// const id = 123442112
// db.receiveCoin(id, 0, '0x1234')
// db.receiveCoin(id, 0, '0x9999')
// db.receiveCoin(id, 1, '0x98973')
// db.receiveCoin(id, 2, '0x6666')
// db.receiveCoin(id+1, 2, '0x6666')
// console.log(db.getCoin(id))
// db.removeCoin(id)
// console.log(db.getCoin(id))
//
// console.log(db.getAllCoins())
//
//# sourceMappingURL=db.js.map